"use strict";
exports.__esModule = true;
exports.unpackPubKey = exports.packPubKey = exports.bigInt2Buffer = exports.SNARK_FIELD_SIZE = exports.NOTHING_UP_MY_SLEEVE = exports.AccQueue = exports.IncrementalQuinTree = exports.formatPrivKeyForBabyJub = exports.unstringifyBigInts = exports.stringifyBigInts = exports.G2Point = exports.G1Point = exports.verifySignature = exports.hashLeftRight = exports.hash13 = exports.hash5 = exports.hash4 = exports.hash3 = exports.hash2 = exports.hashOne = exports.sha256Hash = exports.sign = exports.decrypt = exports.encrypt = exports.genEcdhSharedKey = exports.genKeypair = exports.genPubKey = exports.genPrivKey = exports.genRandomSalt = void 0;
var assert = require("assert");
var crypto = require("crypto");
var ethers = require("ethers");
var ff = require('ffjavascript');
var createBlakeHash = require('blake-hash');
var circomlib_1 = require("circomlib");
var AccQueue_1 = require("./AccQueue");
exports.AccQueue = AccQueue_1.AccQueue;
var optimisedmt_1 = require("optimisedmt");
exports.IncrementalQuinTree = optimisedmt_1.OptimisedMT;
var stringifyBigInts = ff.utils.stringifyBigInts;
exports.stringifyBigInts = stringifyBigInts;
var unstringifyBigInts = ff.utils.unstringifyBigInts;
exports.unstringifyBigInts = unstringifyBigInts;
var G1Point = /** @class */ (function () {
    function G1Point(_x, _y) {
        // TODO: add range checks
        this.x = _x;
        this.y = _y;
    }
    G1Point.prototype.equals = function (pt) {
        return this.x === pt.x && this.y === pt.y;
    };
    G1Point.prototype.asContractParam = function () {
        return {
            x: this.x.toString(),
            y: this.y.toString()
        };
    };
    return G1Point;
}());
exports.G1Point = G1Point;
var G2Point = /** @class */ (function () {
    function G2Point(_x, _y) {
        // TODO: add range checks
        this.x = _x;
        this.y = _y;
    }
    G2Point.prototype.equals = function (pt) {
        return this.x[0] === pt.x[0] &&
            this.x[1] === pt.x[1] &&
            this.y[0] === pt.y[0] &&
            this.y[1] === pt.y[1];
    };
    G2Point.prototype.asContractParam = function () {
        return {
            x: this.x.map(function (n) { return n.toString(); }),
            y: this.y.map(function (n) { return n.toString(); })
        };
    };
    return G2Point;
}());
exports.G2Point = G2Point;
// The BN254 group order p
var SNARK_FIELD_SIZE = BigInt('21888242871839275222246405745257275088548364400416034343698204186575808495617');
exports.SNARK_FIELD_SIZE = SNARK_FIELD_SIZE;
// A nothing-up-my-sleeve zero value
// Should be equal to 8370432830353022751713833565135785980866757267633941821328460903436894336785
var NOTHING_UP_MY_SLEEVE = BigInt(ethers.utils.solidityKeccak256(['bytes'], [ethers.utils.toUtf8Bytes('Maci')])) % SNARK_FIELD_SIZE;
exports.NOTHING_UP_MY_SLEEVE = NOTHING_UP_MY_SLEEVE;
assert(NOTHING_UP_MY_SLEEVE ===
    BigInt('8370432830353022751713833565135785980866757267633941821328460903436894336785'));
/*
 * Convert a BigInt to a Buffer
 */
var bigInt2Buffer = function (i) {
    return Buffer.from(i.toString(16), 'hex');
};
exports.bigInt2Buffer = bigInt2Buffer;
/*
 * Hash an array of uint256 values the same way that the EVM does.
 */
var sha256Hash = function (input) {
    var types = [];
    for (var i = 0; i < input.length; i++) {
        types.push('uint256');
    }
    return BigInt(ethers.utils.soliditySha256(types, input.map(function (x) { return x.toString(); }))) % SNARK_FIELD_SIZE;
};
exports.sha256Hash = sha256Hash;
// Hash up to 2 elements
var poseidonT3 = function (inputs) {
    assert(inputs.length === 2);
    return (0, circomlib_1.poseidon)(inputs);
};
// Hash up to 3 elements
var poseidonT4 = function (inputs) {
    assert(inputs.length === 3);
    return (0, circomlib_1.poseidon)(inputs);
};
// Hash up to 4 elements
var poseidonT5 = function (inputs) {
    assert(inputs.length === 4);
    return (0, circomlib_1.poseidon)(inputs);
};
// Hash up to 5 elements
var poseidonT6 = function (inputs) {
    assert(inputs.length === 5);
    return (0, circomlib_1.poseidon)(inputs);
};
var hashN = function (numElements, elements) {
    var elementLength = elements.length;
    if (elements.length > numElements) {
        throw new TypeError("the length of the elements array should be at most ".concat(numElements, "; got ").concat(elements.length));
    }
    var elementsPadded = elements.slice();
    if (elementLength < numElements) {
        for (var i = elementLength; i < numElements; i++) {
            elementsPadded.push(BigInt(0));
        }
    }
    var funcs = {
        2: poseidonT3,
        3: poseidonT4,
        4: poseidonT5,
        5: poseidonT6
    };
    return funcs[numElements](elements);
};
var hash2 = function (elements) { return hashN(2, elements); };
exports.hash2 = hash2;
var hash3 = function (elements) { return hashN(3, elements); };
exports.hash3 = hash3;
var hash4 = function (elements) { return hashN(4, elements); };
exports.hash4 = hash4;
var hash5 = function (elements) { return hashN(5, elements); };
exports.hash5 = hash5;
/*
 * A convenience function for to use Poseidon to hash a Plaintext with
 * no more than 13 elements
 */
var hash13 = function (elements) {
    var max = 13;
    var elementLength = elements.length;
    if (elementLength > max) {
        throw new TypeError("the length of the elements array should be at most 10; got ".concat(elements.length));
    }
    var elementsPadded = elements.slice();
    if (elementLength < max) {
        for (var i = elementLength; i < max; i++) {
            elementsPadded.push(BigInt(0));
        }
    }
    return poseidonT6([
        elementsPadded[0],
        poseidonT6(elementsPadded.slice(1, 6)),
        poseidonT6(elementsPadded.slice(6, 11)),
        elementsPadded[11],
        elementsPadded[12],
    ]);
};
exports.hash13 = hash13;
/*
 * Hash a single BigInt with the Poseidon hash function
 */
var hashOne = function (preImage) {
    return poseidonT3([preImage, BigInt(0)]);
};
exports.hashOne = hashOne;
/*
 * Hash two BigInts with the Poseidon hash function
 */
var hashLeftRight = function (left, right) {
    return poseidonT3([left, right]);
};
exports.hashLeftRight = hashLeftRight;
/*
 * Returns a BabyJub-compatible random value. We create it by first generating
 * a random value (initially 256 bits large) modulo the snark field size as
 * described in EIP197. This results in a key size of roughly 253 bits and no
 * more than 254 bits. To prevent modulo bias, we then use this efficient
 * algorithm:
 * http://cvsweb.openbsd.org/cgi-bin/cvsweb/~checkout~/src/lib/libc/crypt/arc4random_uniform.c
 * @return A BabyJub-compatible random value.
 */
var genRandomBabyJubValue = function () {
    // Prevent modulo bias
    //const lim = BigInt('0x10000000000000000000000000000000000000000000000000000000000000000')
    //const min = (lim - SNARK_FIELD_SIZE) % SNARK_FIELD_SIZE
    var min = BigInt('6350874878119819312338956282401532410528162663560392320966563075034087161851');
    var rand;
    while (true) {
        rand = BigInt('0x' + crypto.randomBytes(32).toString('hex'));
        if (rand >= min) {
            break;
        }
    }
    var privKey = rand % SNARK_FIELD_SIZE;
    assert(privKey < SNARK_FIELD_SIZE);
    return privKey;
};
/*
 * @return A BabyJub-compatible private key.
 */
var genPrivKey = function () {
    return genRandomBabyJubValue();
};
exports.genPrivKey = genPrivKey;
/*
 * @return A BabyJub-compatible salt.
 */
var genRandomSalt = function () {
    return genRandomBabyJubValue();
};
exports.genRandomSalt = genRandomSalt;
/*
 * An internal function which formats a random private key to be compatible
 * with the BabyJub curve. This is the format which should be passed into the
 * PubKey and other circuits.
 */
var formatPrivKeyForBabyJub = function (privKey) {
    var sBuff = circomlib_1.eddsa.pruneBuffer(createBlakeHash("blake512").update(bigInt2Buffer(privKey)).digest().slice(0, 32));
    var s = ff.utils.leBuff2int(sBuff);
    return ff.Scalar.shr(s, 3);
};
exports.formatPrivKeyForBabyJub = formatPrivKeyForBabyJub;
/*
 * Losslessly reduces the size of the representation of a public key
 * @param pubKey The public key to pack
 * @return A packed public key
 */
var packPubKey = function (pubKey) {
    return circomlib_1.babyJub.packPoint(pubKey);
};
exports.packPubKey = packPubKey;
/*
 * Restores the original PubKey from its packed representation
 * @param packed The value to unpack
 * @return The unpacked public key
 */
var unpackPubKey = function (packed) {
    return circomlib_1.babyJub.unpackPoint(packed);
};
exports.unpackPubKey = unpackPubKey;
/*
 * @param privKey A private key generated using genPrivKey()
 * @return A public key associated with the private key
 */
var genPubKey = function (privKey) {
    // Check whether privKey is a field element
    privKey = BigInt(privKey.toString());
    assert(privKey < SNARK_FIELD_SIZE);
    return circomlib_1.eddsa.prv2pub(bigInt2Buffer(privKey));
};
exports.genPubKey = genPubKey;
var genKeypair = function () {
    var privKey = genPrivKey();
    var pubKey = genPubKey(privKey);
    var Keypair = { privKey: privKey, pubKey: pubKey };
    return Keypair;
};
exports.genKeypair = genKeypair;
/*
 * Generates an Elliptic-Curve Diffie–Hellman (ECDH) shared key given a private
 * key and a public key.
 * @return The ECDH shared key.
 */
var genEcdhSharedKey = function (privKey, pubKey) {
    return circomlib_1.babyJub.mulPointEscalar(pubKey, formatPrivKeyForBabyJub(privKey));
};
exports.genEcdhSharedKey = genEcdhSharedKey;
/*
 * Encrypts a plaintext using a given key.
 * @return The ciphertext.
 */
var encrypt = function (plaintext, sharedKey, nonce) {
    if (nonce === void 0) { nonce = BigInt(0); }
    var ciphertext = (0, circomlib_1.poseidonEncrypt)(plaintext, sharedKey, nonce);
    return ciphertext;
};
exports.encrypt = encrypt;
/*
 * Decrypts a ciphertext using a given key.
 * @return The plaintext.
 */
var decrypt = function (ciphertext, sharedKey, nonce, length) {
    var plaintext = (0, circomlib_1.poseidonDecrypt)(ciphertext, sharedKey, nonce, length);
    return plaintext;
};
exports.decrypt = decrypt;
/*
 * Generates a signature given a private key and plaintext.
 * @return The signature.
 */
var sign = function (privKey, msg) {
    return circomlib_1.eddsa.signPoseidon(bigInt2Buffer(privKey), msg);
};
exports.sign = sign;
/*
 * Checks whether the signature of the given plaintext was created using the
 * private key associated with the given public key.
 * @return True if the signature is valid, and false otherwise.
 */
var verifySignature = function (msg, signature, pubKey) {
    return circomlib_1.eddsa.verifyPoseidon(msg, signature, pubKey);
};
exports.verifySignature = verifySignature;
//# sourceMappingURL=index.js.map